#pragma once

#include "async.h"
#include "io.h"
#include "common.h"

#include <optional>
#include <variant>

namespace saw {
/*
 * It is necessary for the HttpServer to return a HttpStream and not an IoStream.
 * This is due to the nature of Http and the necessity to access the http headers.
 * The whole interface style is retained while not using the inheritance. May be a cause for
 * confusion, but I think that it is mostly acceptable.
 */
class Http {
public:
	struct Method {
		struct Get {};

		using Variant = std::variant<Get>;
	};
};

class HttpStream final : public IoStream {
private:
	Own<IoStream> stream;
public:
	HttpStream(Own<IoStream> stream_);

	SAW_FORBID_COPY(HttpStream);
	SAW_FORBID_MOVE(HttpStream);

	// InputStream
	ErrorOr<size_t> read(void* buffer, size_t length) override;

	Conveyor<void> readReady() override;
	Conveyor<void> onReadDisconnected() override;

	// OutputStream
	ErrorOr<size_t> write(const void* buffer, size_t length) override;

	Conveyor<void> writeReady() override;
};

class HttpServer {
public:
	Conveyor<Own<HttpStream>> accept();
};

class HttpNetwork {
public:
	HttpNetwork(Network& network);

	Own<HttpServer> listen(NetworkAddress& address);
	Conveyor<Own<HttpStream>> connect(NetworkAddress& address);
private:
	Network* internal;
};

/*
 * Wrappers around the base classes describing Http behaviour
 */

class HttpResponse {

};

/*
 * A lifetime object. Request is already done, but 
 */
class HttpRequest {
public:
	HttpRequest(Own<ConveyorFeeder>

	Conveyor<HttpResponse> response();
private:
	Own<ConveyorFeeder<HttpResponse>> feeder;

};

class HttpClient {
public:
	Own<HttpRequest> request(Http::Method::Variant method, const HttpNetworkAddress& address);


};
}
